/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.codegen;

import java.util.List;

import org.eclipse.emf.ecore.EObject;

import com.ibm.ccl.mapping.ConditionRefinement;
import com.ibm.ccl.mapping.CustomFunctionRefinement;
import com.ibm.ccl.mapping.FunctionRefinement;
import com.ibm.ccl.mapping.GroupRefinement;
import com.ibm.ccl.mapping.Mapping;
import com.ibm.ccl.mapping.MappingDesignator;
import com.ibm.ccl.mapping.MappingRoot;
import com.ibm.ccl.mapping.SortDesignator;
import com.ibm.ccl.mapping.SubmapRefinement;

/**
 * A handler to access mapping information.
 * @author David Lauzon
 */
public interface MappingHandler
{
  /**
   * TypeInfo values of an EObject.
   * 
   * SIMPLE: contains data content only.
   * COMPLEX_WITH_STRUCTURED_CONTENT: contains structural content only.
   * COMPLEX_WITH_MIXED_CONTENT: contains data and structural content.
   */
  /** @deprecated use XSDEcoreConstants.SIMPLE */
  public final static int SIMPLE = 0;
  /** @deprecated use XSDEcoreConstants.COMPLEX_WITH_SIMPLE_CONTENT */
  public final static int COMPLEX_WITH_SIMPLE_CONTENT = 1;
  /** @deprecated use XSDEcoreConstants.STRUCTURED_CONTENT */
  public final static int COMPLEX_WITH_STRUCTURED_CONTENT = 2;
  /** @deprecated use XSDEcoreConstants.COMPLEX_WITH_MIXED_CONTENT */
  public final static int COMPLEX_WITH_MIXED_CONTENT = 3;
	  
  /**
   * MappingInfo values of a target object (EObject).
   * 
   * NO_MAPPING: the target is not mapped nor does it have any mapped content.
   * NO_MAPPING_WITH_MAPPED_CONTENT: the target itself is not mapped but it does have mapped content.
   * SINGLE_MAPPING: the target is mapped to a single source.
   * SINGLE_MAPPING_WITH_MULTIPLE_INPUTS: the target is mapped to multiple sources.
   * MULTIPLE_MAPPINGS: the target has multiple mappings defined on it.
   */
  /** @deprecated use MappingConstants.NO_MAPPING */
  public final static int NO_MAPPING = 0;
  /** @deprecated use MappingConstants.NO_MAPPING_WITH_MAPPED_CONTENT */  
  public final static int NO_MAPPING_WITH_MAPPED_CONTENT = 1;
  /** @deprecated use MappingConstants.SINGLE_MAPPING */  
  public final static int SINGLE_MAPPING = 2;
  /** @deprecated use MappingConstants.SINGLE_MAPPING_WITH_MULTIPLE_INPUTS */  
  public final static int SINGLE_MAPPING_WITH_MULTIPLE_INPUTS = 3;
  /** @deprecated use MappingConstants.MULTIPLE_MAPPINGS */  
  public final static int MULTIPLE_MAPPINGS = 4;	

  /**
   * Return the type infomation for the given EObject.
   * @param eobject an EObject object.
   * @return the type infomation for the given type.
   * @deprecated use XSDEcoreUtils.getTypeInfo(EObject eobject).
   */
  public int getTypeInfo(EObject eobject);

  /**
   * Return the mapping status of the target object.
   * @param mapping of the target object.
   * @param eobject the target object.
   * @return the mapping status of the target object.
   * @deprecated use RuleInfo.getMappingInfo(List mappings).
   */
  public int getMappingInfo(Mapping baseMapping, EObject target);
  
  /**
   * Return the mapping status of the target object.
   * @param mappings of the target object.
   * @param eobject the target object.
   * @return the mapping status of the target object.
   * @deprecated use RuleInfo.getMappingInfo(List mappings).
   */
  public int getMappingInfo(List nestedMappings, EObject target);

  /**
   * Return the mapping root corresponding to this handler.
   * @return the mapping root.
   */
  public MappingRoot getMappingRoot();
	
  /**
   * Set the mapping root for this handler.
   * @param mappingRoot the mapping root.
   */
  public void setMappingRoot(MappingRoot mappingRoot);

  /**
   * Return all of the mappings associated with this handler.
   * @return all of the mappings.
   */
  public List getAllMappings();

  /**
   * Return all of the top-level mappings associated with this handler.
   * @return all of the top-level mappings.
   */
  public List getTopLevelMappings();

  /**
   * Return a list of nested mappings for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of nested mappings.
   * @deprecated use ModelUtils.getMappingRoot().
   */
  public List getNestedMappings(Mapping mapping);

  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param context the current mapping context
   * @param ebbject the target object.
   * @return the mapping associated with the target object.
   * @deprecated use ModelUtils.getMapping(Mapping context, EObject eobject).
   */
  public Mapping getMapping(Mapping context, EObject eobject);

  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mapping associated with the target object.
   * @deprecated use ModelUtils.getMapping(List nestedMappings, EObject eobject).
   */
  public Mapping getMapping(List nestedMappings, EObject eobject);

  /**
   * Return the mappings associated with the target object.
   * 
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mappings associated with the target object.
   * @deprecated use ModelUtils.getMappings(List nestedMappings, EObject eobject, Stack mappingDeclarationContext).
   */
  public List getMappings(List nestedMappings, EObject eobject);

  /**
   * Returns true if the specified mapping is a element reference mapping.
   * A mapping is considered an element reference mapping:
   *    1. if the mapping is a submap refinement and
   *    2. both the inputs and outputs contain a single element
   *
   * @param mapping a Mapping object.
   * @return true if the specified mapping is a reference mapping.
   * @deprecated use XSDEcoreUtils.isElementMappingReference(Mapping mapping).
   */
  public boolean isElementMappingReference(Mapping mapping);
	
  /**
   * Returns true if the specified mapping is a type reference mapping.
   * A mapping is a considered a type reference if:
   *    1. the mapping contains a submap refinement and
   *    2. at least one of the inputs or outputs is a type. 
   *    
   * @param mapping a Mapping object.
   * @return true if the specified mapping is a type reference mapping.
   * @deprecated use XSDEcoreUtils.isTypeMappingReference(Mapping mapping).
   */
  public boolean isTypeMappingReference(Mapping mapping);

  /**
   * Return the submap refinement for the specified mapping.
   * @param mapping a Mapping object.
   * @return the submap refinement for the specified mapping.
   * @deprecated use ModelUtils.getSubmapRefinement(Mapping mapping).
   */
  public SubmapRefinement getSubmapRefinement(Mapping mapping);

  /**
   * Return the ref name of the submap refinement 
   * @param submap a submap refinement.
   * @return the ref name of the submap refinement.
   * @deprecated use ModelUtils.getRefName(SubmapRefinement submap).
   */
  public String getRefName(SubmapRefinement submap);
	
  /**
   * Return a list of the source packages associated with this handler.
   * @return a list of the source packages.
   */
  public List getSourcePackages();
	
  /**
   * Return a list of the target packages associated with this handler.
   * @return a list of the target packages.
   */
  public List getTargetPackages();
	
  /**
   * Return the sources for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of sources for the specified mapping.
   * @deprecated use ModelUtils.getSources(Mapping mapping).
  */
  public List getSources(Mapping mapping);
	
  /**
   * Return the target for the specified mapping.
   * Note if multiple target objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   * @deprecated use ModelUtils.getPrimaryTarget(Mapping mapping).
   */
  public EObject getPrimaryTarget(Mapping mapping);

  /**
   * Return the source for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   * @deprecated use ModelUtils.getPrimarySource(Mapping mapping).
   */
  public EObject getPrimarySource(Mapping mapping);

  /**
   * Return the target for the specified mapping.
   * @param mapping a Mapping object.
   * @return the target for the specified mapping.
   * @deprecated use ModelUtils.getTarget(Mapping mapping).
   */
  public EObject getTarget(Mapping mapping);

  /**
   * Return the source designators for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of source designators for the specified mapping.
   * @deprecated use ModelUtils.getSourceDesignators(Mapping mapping).
   */
  public List getSourceDesignators(Mapping mapping);
	
  /**
   * Return the target designator for the specified mapping.
   * Note if multiple target objects, the first target object is returned.
   *    
   * @param mapping a Mapping object.
   * @return the target designator for the specified mapping.
   * @deprecated use ModelUtils.getPrimaryTargetDesignator(Mapping mapping).
   */
  public MappingDesignator getPrimaryTargetDesignator(Mapping mapping);

  /**
   * Return the source designator for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source designator for the specified mapping.
   * @deprecated use ModelUtils.getPrimarySourceDesignator(Mapping mapping).
   */
  public MappingDesignator getPrimarySourceDesignator(Mapping mapping);

  /**
   * Return the base designator for the specified mapping that is the 
   * ancestor of the specified designator.
   * 
   * @param mapping a Mapping object.
   * @param designator a descendant of the base designator.
   * @return the source designator for the specified mapping.
   * @deprecated use ModelUtils.getBaseSourceDesignator(Mapping mapping, MappingDesignator designator).
   */
  public MappingDesignator getBaseSourceDesignator(Mapping mapping, MappingDesignator designator);
  
  /**
   * Returns the type of the specified EObject.
   * @param eobject an EObject.
   * @return the type of the specified EObject.
   * @deprecated use XSDEcoreUtils.getType(EObject eobject);
   */
  public EObject getType(EObject eobject);
 
  /**
   * Returns true if the designator represents a type.
   * @param designator a MappingDesignator object.
   * @return true if the designator represents a type.
   * @deprecated use XSDEcoreUtils.isType(MappingDesignator designator);
   */
  public boolean isType(MappingDesignator designator);

  /**
   * Returns the name of the specified EObject.
   * @param eobject an EObject.
   * @return the name of the specified EObject.
   * @deprecated use XSDEcoreUtils.getTypeName(EObject eobject);
   */
  public String getTypeName(EObject eobject);

  /**
   * Returns true if the specified eobject can occur multiple times within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject can occur multiple times.
   * @deprecated use XSDEcoreUtils.isRepeatable(EObject eobject).
   */
  public boolean isRepeatable(EObject eobject);	

  /**
   * Returns true if the specified eobject can occur multiple times within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject can occur multiple times.
   * @deprecated use XSEcoreUtils.isRepeatableDesignator(MappingDesignator designator).
   */
  public boolean isRepeatableDesignator(MappingDesignator designator);	

  /**
   * Returns true if the specified eobject is optional within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject is optional.
   * @deprecated use XSDEcoreUtils.isOptional(EObject eobject).
   */
  public boolean isOptional(EObject eobject);


  /**
   * Returns true if the specified mapping contains a condition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a condition.
   * @deprecated use ModelUtils.hasConditionRefinement(Mapping mapping).
   */
  public boolean hasCondition(Mapping mapping);

  /**
   * Returns a  condition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a condition for the specified mapping.
   * @deprecated use ModelUtils.getConditionRefinement(Mapping mapping)
   */
  public ConditionRefinement getCondition(Mapping mapping);

  /**
   * Returns the test value of the specified condition.
   * @param condition a ConditionRefinement object.
   * @return the test value of the specified condition.
   * @deprecated use ModelUtils.getTestValue(ConditionRefinement condition).
   */
  public String getTestValue(ConditionRefinement condition);
	
  /**
   * Returns true if the specified mapping contains inner scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains inner scope conditions.
   * @deprecated use ModelUtils.hasInnerScopeCondition(Mapping mapping).
   */
  public boolean hasInnerScopeCondition(Mapping mapping);

  /**
   * Returns a list of inner scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope conditions for the specified mapping.
   * @deprecated use ModelUtils.getInnerScopeCondition(Mapping mapping).
   */
  public ConditionRefinement getInnerScopeCondition(Mapping mapping);
  
  /**
   * Returns true if the specified mapping contains outer scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains outer scope conditions.
   * @deprecated use ModelUtils.hasOuterScopeCondition(Mapping mapping).
   */
  public boolean hasOuterScopeCondition(Mapping mapping);

  /**
   * Returns a list of outer scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope conditions for the specified mapping.
   * @deprecated use ModelUtils.hasOuterScopeCondition(Mapping mapping).
   */
  public ConditionRefinement getOuterScopeCondition(Mapping mapping);

  /**
   * Returns true if the specified mapping contains grouping definitions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains grouping definitions.
   * @deprecated use ModelUtils.hasGroupRefinement(Mapping mapping);
   */
  public boolean hasGrouping(Mapping mapping);

  /**
   * Returns a Grouping definition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a Grouping definition for the specified mapping.
   * @deprecated use ModelUtils.getGroupRefinement(Mapping mapping).
   */
  public GroupRefinement getGrouping(Mapping mapping);

  /**
   * Returns the name for the specified MSLGrouping object.
   * @param grouping a MSLGrouping object.
   * @return the name for the specified MSLGrouping object.
   * @deprecated use ModelUtils.getGroupingName(GroupRefinement grouping).
   */
  public String getGroupingName(GroupRefinement grouping);

  /**
   * Returns true if the specified mapping contains a function.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains  function.
   * @deprecated use ModelUtils.hasFunction(Mapping mapping);
   */
  public boolean hasFunction(Mapping mapping);

  /**
   * Returns the function for the specified mapping.
   * @param mapping  a Mapping object.
   * @return the function for the specified mapping.
   * @deprecated use ModelUtils.getFunction(Mapping mapping).
   */
  public FunctionRefinement getFunction(Mapping mapping);

  /**
   * Returns true if the specified mapping contains a Sort definition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a Sort definition.
   * @deprecated use ModelUtils.hasSortRefinement(Mapping mapping);
   */
  public boolean hasSortDefined(Mapping mapping);

  /**
   * Returns a list of sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of sorts for the specified mapping.
   * @deprecated use ModelUtils.getSorts(Mapping mapping).
   */
  public List getSorts(Mapping mapping);

   /**
   * Returns a list of inner scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope sorts for the specified mapping.
   * @deprecated use ModelUtils.getInnerScopeSorts(Mapping mapping).
   */
  public List getInnerScopeSorts(Mapping mapping);
	
  /**
   * Returns a list of outer scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope sorts for the specified mapping.
   * @deprecated use ModelUtils.getOuterScopeSorts(Mapping mapping).
   */
  public List getOuterScopeSorts(Mapping mapping);
	
  /**
   * Returns the order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the order value for the specified MSLSort object.
   * @deprecated use ModelUtils.getOrder(SortDesignator sort).
   */
  public String getOrder(SortDesignator sort);

  /**
   * Returns the case-order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the case-order value for the specified MSLSort object.
   * @deprecated use ModelUtils.getCaseOrder(SortDesignator sort).
   */
  public String getCaseOrder(SortDesignator sort);

  /**
   * Returns the data-type value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the data-type value for the specified MSLSort object.
   * @deprecated use ModelUtils.getDataType(SortDesignator sort).
   */
  public String getDataType(SortDesignator sort);

  /**
   * Retrieves the custom function refinement from the given mapping 
   * if the mapping has one.
   * @param mapping a Mapping object.
   * @return a CustomFunctionRefinement if the mapping has one or null otherwise.
   * @deprecated use ModelUtils.getCustomRefinement(Mapping mapping).
   */
  public CustomFunctionRefinement getCustomRefinement(Mapping mapping);
  
  /**
   * Determines if a custom refinement's code is external or not.
   * @param custom the custom refinement to test.
   * @return true if a custom refinement's code is external, false otherwise.
   * @deprecated use ModelUtils.isExternal(CustomFunctionRefinement custom).
   */
  public boolean isExternal(CustomFunctionRefinement custom);
}