/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.ccl.mapping.codegen.impl;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.ExtendedMetaData;

import com.ibm.ccl.mapping.ConditionRefinement;
import com.ibm.ccl.mapping.CustomFunctionRefinement;
import com.ibm.ccl.mapping.FunctionRefinement;
import com.ibm.ccl.mapping.GroupRefinement;
import com.ibm.ccl.mapping.Mapping;
import com.ibm.ccl.mapping.MappingDeclaration;
import com.ibm.ccl.mapping.MappingDesignator;
import com.ibm.ccl.mapping.MappingRoot;
import com.ibm.ccl.mapping.SortDesignator;
import com.ibm.ccl.mapping.SubmapRefinement;
import com.ibm.ccl.mapping.codegen.MappingHandler;
import com.ibm.ccl.mapping.codegen.util.ModelUtils;
import com.ibm.ccl.mapping.resolvers.IPathResolver;

/**
 * A handler to access mapping information.
 * @author David Lauzon
 * 
 */

public abstract class MappingHandlerImpl implements MappingHandler 
{
  protected MappingRoot mappingRoot;
  protected IPathResolver pathResolver;
  protected List sourcePackages = Collections.EMPTY_LIST;
  protected List targetPackages = Collections.EMPTY_LIST;

  /**
   * Return the type infomation for the given EObject.
   * @param eobject an EObject object.
   * @return the type infomation for the given type.
   * @deprecated use XSDEcoreUtils.getTypeInfo(EObject eobject).
   */
  public int getTypeInfo(EObject eobject)
  {
    int result = SIMPLE;
    if (eobject instanceof EClass)
    {
      if (ExtendedMetaData.INSTANCE.getContentKind((EClass)eobject) == ExtendedMetaData.SIMPLE_CONTENT)
      {
    	result = COMPLEX_WITH_SIMPLE_CONTENT;
      }
      else
      {
        result = COMPLEX_WITH_STRUCTURED_CONTENT;
      }
    }
    return result;
  }

  /**
   * Return the mapping status of the target object.
   * @param mapping of the target object.
   * @param eobject the target object.
   * @return the mapping status of the target object.
   * @deprecated use RuleInfo.getMappingInfo(List mappings).
   */
  public abstract int getMappingInfo(Mapping baseMapping, EObject target);
  
  /**
   * Return the mapping status of the target object.
   * @param mappings of the target object.
   * @param eobject the target object.
   * @return the mapping status of the target object.
   * @deprecated use RuleInfo.getMappingInfo(List mappings).
   */
  public abstract int getMappingInfo(List nestedMappings, EObject target);

  /* (non-Javadoc)
   * @see com.ibm.etools.msl.impl.MappingHandler#getMappingRoot()
   */
  public MappingRoot getMappingRoot()
  {
    return mappingRoot;
  }

  /* (non-Javadoc)
   * @see com.ibm.etools.msl.impl.MappingHandler#setMappingRoot(com.ibm.etools.msl.MappingRoot)
   */
  public void setMappingRoot(MappingRoot mappingRoot)
  {
    this.mappingRoot = mappingRoot;
    if (mappingRoot != null)
    {
      sourcePackages = getPackages(mappingRoot.getInputs());
      targetPackages = getPackages(mappingRoot.getOutputs());
      pathResolver = (IPathResolver)EcoreUtil.getRegisteredAdapter(mappingRoot, IPathResolver.class);
    }
  }
  
  
  private List getPackages(List designators) 
  {
	List result = new ArrayList();
	if (designators != null)
    for (Iterator i = designators.iterator(); i.hasNext();)
	{
      MappingDesignator designator = (MappingDesignator)i.next();
      Object obj = designator.getObject();
      if (obj instanceof EPackage)
      {
        result.add((EPackage)obj);
      }
	}
	return result;
  }

  /* (non-Javadoc)
   * @see com.ibm.etools.msl.impl.MappingHandler#getAllMappings()
   */
  public List getAllMappings()
  {
  	List result = new ArrayList();
  	List mappings = getTopLevelMappings();
  	if (mappings != null)
  	{
	  for (Iterator i= mappings.iterator(); i.hasNext();)
  	  {
  		Mapping mapping = (Mapping)i.next();
  		result.add(mapping);
  		List nestedMappings = getNestedMappings(mapping);
  		if (nestedMappings != null)
  		{
  			result.addAll(nestedMappings);
  		}
  	  }
  	}
  	return result;
  }

  
  /* (non-Javadoc)
   * @see com.ibm.etools.msl.impl.MappingHandler#getTopLevelMappings()
   */
  public List getTopLevelMappings()
  {
    List list = new ArrayList();
    
    List childMappings = this.mappingRoot.getNested();

	for (Iterator i = childMappings.iterator(); i.hasNext(); )
	{
    Object o = i.next();
	  if (o instanceof Mapping)
	  {  
	  	list.add((Mapping)o);
	  }  
	}  
	
	return list;
 }
  
  /**
   * @deprecated use ModelUtils.getInputMappings(Mapping, EObject).
   */
  public List getInputMappings(Mapping baseMapping, EObject target)
  {
	 return ModelUtils.getInputMappings(baseMapping, target);
  }

  /**
   * @deprecated use ModelUtils.getInputMappings(Mapping, EObject, EObject).
   */
  public List getInputMappings(Mapping baseMapping, EObject ancestor, EObject target)
  {
	return ModelUtils.getInputMappings(baseMapping, ancestor, target);
  }
  
  /**
   * @deprecated use ModelUtils.isCommonAncestor(MappingDesignator, EObject).
   */
  protected boolean isCommonAncestor(MappingDesignator designator, EObject ancestor) 
  {
	return ModelUtils.isCommonAncestor(designator, ancestor);
  }

  /**
   * Return a list of nested mappings for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of nested mappings.
   * @deprecated use ModelUtils.getMappingRoot().
   */
  public List getNestedMappings(Mapping mapping)
  {
    return ModelUtils.getNestedMappings(mapping);
  }


  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param context the current mapping context
   * @param ebbject the target object.
   * @return the mapping associated with the target object.
   * @deprecated use ModelUtils.getMapping(Mapping context, EObject eobject).
   */
  public Mapping getMapping(Mapping context, EObject eobject)
  {
	return ModelUtils.getMapping(context, eobject);
  }
  
  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mapping associated with the target object.
   * @deprecated use ModelUtils.getMapping(List nestedMappings, EObject eobject).
   */
  public Mapping getMapping(List nestedMappings, EObject eobject)
  {
    return ModelUtils.getMapping(nestedMappings, eobject);
  }

  /**
   * Return the mappings associated with the target object.
   * 
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mappings associated with the target object.
   * @deprecated use ModelUtils.getMappings(List nestedMappings, EObject eobject, Stack mappingDeclarationContext).
   */
  public List getMappings(List nestedMappings, EObject eobject)
  {
	List result = new ArrayList();
	if (nestedMappings != null)
	{
	  int size = nestedMappings.size()-1;
	  for (int i = size; i>=0; i--)
	  {
	  	Mapping contextMapping = (Mapping)nestedMappings.get(i);
	   	List mappings = ModelUtils.getInputMappings(contextMapping, eobject);
	    if ((mappings != null) && (!mappings.isEmpty()))
	    {
	      result = mappings;
	      break;
	    }
	  }
	}
	return result;
  }

  
  /**
   * Returns true if the specified mapping is a element reference mapping.
   * A mapping is considered an element reference mapping:
   *    1. if the mapping is a submap refinement and
   *    2. both the inputs and outputs contain a single element
   *
   * @param mapping a Mapping object.
   * @return true if the specified mapping is a reference mapping.
   * @deprecated use XSDEcoreUtils.isElementMappingReference(Mapping mapping).
   */
  public boolean isElementMappingReference(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  SubmapRefinement refinement = ModelUtils.getSubmapRefinement(mapping);
	  if (refinement != null)
	  {
	    MappingDeclaration mappingDeclaration = ((SubmapRefinement)refinement).getRef();
	    if (mappingDeclaration != null)
	 	{
	      List inputs = mappingDeclaration.getInputs();
	 	  List outputs = mappingDeclaration.getOutputs();
	 	  result = ((!inputs.isEmpty()) && (!isType((MappingDesignator)inputs.get(0))) &&
	 		        (!outputs.isEmpty()) && (!isType((MappingDesignator)outputs.get(0))));
	 	}
	  }
	}
	return result;
  }
  
  /**
   * Returns true if the specified mapping is a type reference mapping.
   * A mapping is a considered a type reference if:
   *    1. the mapping contains a submap refinement and
   *    2. at least one of the inputs or outputs is a type. 
   *    
   * @param mapping a Mapping object.
   * @return true if the specified mapping is a type reference mapping.
   * @deprecated use XSDEcoreUtils.isTypeMappingReference(Mapping mapping).
   */
  public boolean isTypeMappingReference(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  SubmapRefinement refinement = ModelUtils.getSubmapRefinement(mapping);
	  if (refinement != null)
	  {
	    MappingDeclaration mappingDeclaration = ((SubmapRefinement)refinement).getRef();
	    if (mappingDeclaration != null)
		{
	      boolean inputsContainType = false;
	      for (Iterator i = mappingDeclaration.getInputs().iterator(); i.hasNext();)
	      {
	        if (isType((MappingDesignator)i.next())) 
	 	    {
	  	      inputsContainType = true;
	  	      break;
	  	    }
	  	  }
	  	  List outputs = mappingDeclaration.getOutputs();
	  	  result = inputsContainType || ((!outputs.isEmpty()) && isType((MappingDesignator)outputs.get(0)));
	  	}
	  }
	}
	return result;
  }

  /**
   * Return the submap refinement for the specified mapping.
   * @param mapping a Mapping object.
   * @return the submap refinement for the specified mapping.
   * @deprecated use ModelUtils.getSubmapRefinement(Mapping mapping).
   */
  public SubmapRefinement getSubmapRefinement(Mapping mapping)
  {
  	return ModelUtils.getSubmapRefinement(mapping);
  }

  /**
   * Return the ref name of the submap refinement 
   * @param submap a submap refinement.
   * @return the ref name of the submap refinement.
   * @deprecated use ModelUtils.getRefName(SubmapRefinement submap).
   */
  public String getRefName(SubmapRefinement submap)
  {
	return ModelUtils.getRefName(submap);
  }
  
  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.codegen.MappingHandler#getSourcePackages()
   */
  public List getSourcePackages()
  {
	return this.sourcePackages;
  }
  
  /* (non-Javadoc)
   * @see com.ibm.ccl.mapping.codegen.MappingHandler#getTargetPackages()
   */
  public List getTargetPackages()
  {
	return this.targetPackages;
  }
  
  /**
   * Return the sources for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of sources for the specified mapping.
   * @deprecated use ModelUtils.getSources(Mapping mapping).
   */
  public List getSources(Mapping mapping)
  {
	  return ModelUtils.getSources(mapping);
  }


  /**
   * Return the target for the specified mapping.
   * Note if multiple target objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   * @deprecated use ModelUtils.getPrimaryTarget(Mapping mapping).
   */
  public EObject getPrimaryTarget(Mapping mapping)
  {
    return ModelUtils.getPrimaryTarget(mapping);
  }

  
  /**
   * Return the source for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   * @deprecated use ModelUtils.getPrimarySource(Mapping mapping).
   */
  public EObject getPrimarySource(Mapping mapping)
  {
    return ModelUtils.getPrimarySource(mapping);
  }

  
  /**
   * Return the target for the specified mapping.
   * @param mapping a Mapping object.
   * @return the target for the specified mapping.
   * @deprecated use ModelUtils.getTarget(Mapping mapping).
   */
  public EObject getTarget(Mapping mapping)
  {
    return ModelUtils.getTarget(mapping);
  }

  
  /**
   * Return the source designators for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of source designators for the specified mapping.
   * @deprecated use ModelUtils.getSourceDesignators(Mapping mapping).
   */
  public List getSourceDesignators(Mapping mapping)
  {
    return ModelUtils.getSourceDesignators(mapping);
  }

  /**
   * Return the target designator for the specified mapping.
   * Note if multiple target objects, the first target object is returned.
   *    
   * @param mapping a Mapping object.
   * @return the target designator for the specified mapping.
   * @deprecated use ModelUtils.getPrimaryTargetDesignator(Mapping mapping).
   */
  public MappingDesignator getPrimaryTargetDesignator(Mapping mapping)
  {
	return ModelUtils.getPrimaryTargetDesignator(mapping);
  }
  
  /**
   * Return the source designator for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source designator for the specified mapping.
   * @deprecated use ModelUtils.getPrimarySourceDesignator(Mapping mapping).
   */
   public MappingDesignator getPrimarySourceDesignator(Mapping mapping)
  {
    return ModelUtils.getPrimarySourceDesignator(mapping);
  }

  /**
   * Return the base designator for the specified mapping that is the 
   * ancestor of the specified designator.
   * 
   * @param mapping a Mapping object.
   * @param designator a descendant of the base designator.
   * @return the source designator for the specified mapping.
   * @deprecated use ModelUtils.getBaseSourceDesignator(Mapping mapping, MappingDesignator designator).
   */
  public MappingDesignator getBaseSourceDesignator(Mapping mapping, MappingDesignator designator)
  {
    return ModelUtils.getBaseSourceDesignator(mapping, designator);
  }
  
  /**
   * Returns the type of the specified EObject.
   * @param eobject an EObject.
   * @return the type of the specified EObject.
   * @deprecated use XSDEcoreUtils.getType(EObject eobject);
   */
  public EObject getType(EObject eobject)
  {
	  EObject result = null;
	  if (eobject instanceof EStructuralFeature)
	  {
	    result = ((EStructuralFeature)eobject).getEType();
	  }
	  else if (eobject instanceof EClassifier)
	  {
		 result = eobject;
	  }
	  return result;
  }

  /**
   * Returns true if the designator represents a type.
   * @param designator a MappingDesignator object.
   * @return true if the designator represents a type.
   * @deprecated use XSDEcoreUtils.isType(MappingDesignator designator);
   */
  public boolean isType(MappingDesignator designator)
  {
	  boolean result = false;
	  if (designator != null)
	  {
	    result = (designator.getObject() instanceof EClassifier);
	  }
	  return result;
  }

  /**
   * Returns the name of the specified EObject.
   * @param eobject an EObject.
   * @return the name of the specified EObject.
   * @deprecated use XSDEcoreUtils.getTypeName(EObject eobject);
   */
  public String getTypeName(EObject eobject)
  {
	  String result = null;
	  if (eobject instanceof EStructuralFeature)
	  {
	    result = ((EStructuralFeature)eobject).getEType().getName();
	  }
	  else if (eobject instanceof EClassifier)
	  {
		result = ((EClassifier)eobject).getName();
	  }
	  return result;
  }
  
  /**
   * Returns true if the specified eobject can occur multiple times within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject can occur multiple times.
   * @deprecated use XSDEcoreUtils.isRepeatable(EObject eobject).
   */
  public boolean isRepeatable(EObject eobject)
  {
  	boolean result = false;
  	if (eobject instanceof EStructuralFeature)
  	{
	    result = (((EStructuralFeature)eobject).getUpperBound() > 1 || 
	    		      ((EStructuralFeature)eobject).getUpperBound() == -1);
  	}
  	return result;
  }

  /**
   * Returns true if the specified eobject can occur multiple times within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject can occur multiple times.
   * @deprecated use XSEcoreUtils.isRepeatableDesignator(MappingDesignator designator).
   */
  public boolean isRepeatableDesignator(MappingDesignator designator)
  {
  	boolean result = false;
  	if (designator != null)
  	{
      if (isRepeatable(designator.getObject()))
      {
  	    result = ((designator.getIndex() == null) || (designator.getIndex().length() == 0));
      }
  	}
  	return result;
  }

  /**
   * Returns true if the specified eobject is optional within its context.
   * @param eobject an EObject.
   * @return true if the specified eobject is optional.
   * @deprecated use XSDEcoreUtils.isOptional(EObject eobject).
   */
  public boolean isOptional(EObject eobject)
  {
  	boolean result = true;
  	if (eobject instanceof EStructuralFeature)
  	{
  		result = (((EStructuralFeature)eobject).getLowerBound()== 0) && 
  		         (((EStructuralFeature)eobject).getUpperBound()== 1);
  	}
  	return result;
  }

  
  /**
   * Returns true if the specified mapping contains a condition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a condition.
   * @deprecated use ModelUtils.hasConditionRefinement(Mapping mapping).
   */
  public boolean hasCondition(Mapping mapping)
  {
    return ModelUtils.hasConditionRefinement(mapping);
  }


  /**
   * Returns a  condition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a condition for the specified mapping.
   * @deprecated use ModelUtils.getConditionRefinement(Mapping mapping)
   */
  public ConditionRefinement getCondition(Mapping mapping)
  {
    return ModelUtils.getConditionRefinement(mapping);
  }

  /**
   * Returns the test value of the specified condition.
   * @param condition a ConditionRefinement object.
   * @return the test value of the specified condition.
   * @deprecated use ModelUtils.getTestValue(ConditionRefinement condition).
   */
  public String getTestValue(ConditionRefinement condition)
  {
	return ModelUtils.getTestValue(condition);
  }

  /**
   * Returns true if the specified mapping contains inner scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains inner scope conditions.
   * @deprecated use ModelUtils.hasInnerScopeCondition(Mapping mapping).
   */
  public boolean hasInnerScopeCondition(Mapping mapping)
  {
    return ModelUtils.hasInnerScopeCondition(mapping);
  }


  /**
   * Returns a list of inner scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope conditions for the specified mapping.
   * @deprecated use ModelUtils.getInnerScopeCondition(Mapping mapping).
   */
  public ConditionRefinement getInnerScopeCondition(Mapping mapping)
  {
    return ModelUtils.getInnerScopeCondition(mapping);
  }

  
  /**
   * Returns true if the specified mapping contains outer scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains outer scope conditions.
   * @deprecated use ModelUtils.hasOuterScopeCondition(Mapping mapping).
   */
  public boolean hasOuterScopeCondition(Mapping mapping)
  {
	return ModelUtils.hasOuterScopeCondition(mapping);
  }


  /**
   * Returns a list of outer scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope conditions for the specified mapping.
   * @deprecated use ModelUtils.hasOuterScopeCondition(Mapping mapping).
   */
  public ConditionRefinement getOuterScopeCondition(Mapping mapping)
  {
    return ModelUtils.getOuterScopeCondition(mapping);
  }


  /**
   * Returns true if the specified mapping contains grouping definitions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains grouping definitions.
   * @deprecated use ModelUtils.hasGroupRefinement(Mapping mapping);
   */
  public boolean hasGrouping(Mapping mapping)
  {
    return ModelUtils.hasGroupRefinement(mapping);
  }

  
  /**
   * Returns a Grouping definition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a Grouping definition for the specified mapping.
   * @deprecated use ModelUtils.getGroupRefinement(Mapping mapping).
   */
  public GroupRefinement getGrouping(Mapping mapping)
  {
    return ModelUtils.getGroupRefinement(mapping);
  }


  /**
   * Returns the name for the specified MSLGrouping object.
   * @param grouping a MSLGrouping object.
   * @return the name for the specified MSLGrouping object.
   * @deprecated use ModelUtils.getGroupingName(GroupRefinement grouping).
   */
  public String getGroupingName(GroupRefinement grouping)
  {
   return ModelUtils.getGroupingName(grouping);
  }


  /**
   * Returns true if the specified mapping contains a function.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains  function.
   * @deprecated use ModelUtils.hasFunction(Mapping mapping);
   */
  public boolean hasFunction(Mapping mapping)
  {
    return ModelUtils.hasFunctionRefinement(mapping);
  }

  
  /**
   * Returns the function for the specified mapping.
   * @param mapping  a Mapping object.
   * @return the function for the specified mapping.
   * @deprecated use ModelUtils.getFunction(Mapping mapping).
   */
  public FunctionRefinement getFunction(Mapping mapping)
  {
    return ModelUtils.getFunctionRefinement(mapping);
  }

  /**
   * Returns true if the specified mapping contains a Sort definition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a Sort definition.
   * @deprecated use ModelUtils.hasSortRefinement(Mapping mapping);
   */
  public boolean hasSortDefined(Mapping mapping)
  {
    return ModelUtils.hasSortRefinement(mapping);
  }

  /**
   * Returns a list of sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of sorts for the specified mapping.
   * @deprecated use ModelUtils.getSorts(Mapping mapping).
   */
  public List getSorts(Mapping mapping)
  {
    return ModelUtils.getSorts(mapping);
  }

  /**
   * Returns a list of inner scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope sorts for the specified mapping.
   * @deprecated use ModelUtils.getInnerScopeSorts(Mapping mapping).
   */
  public List getInnerScopeSorts(Mapping mapping)
  {
    return ModelUtils.getInnerScopeSorts(mapping);
  }

  
  /**
   * Returns a list of outer scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope sorts for the specified mapping.
   * @deprecated use ModelUtils.getOuterScopeSorts(Mapping mapping).
   */
  public List getOuterScopeSorts(Mapping mapping)
  {
    return ModelUtils.getOuterScopeSorts(mapping);
  }


  /**
   * Returns the order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the order value for the specified MSLSort object.
   * @deprecated use ModelUtils.getOrder(SortDesignator sort).
   */
  public String getOrder(SortDesignator sort)
  {
    return ModelUtils.getOrder(sort);
  }

  /**
   * Returns the case-order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the case-order value for the specified MSLSort object.
   * @deprecated use ModelUtils.getCaseOrder(SortDesignator sort).
   */
  public String getCaseOrder(SortDesignator sort)
  {
    return ModelUtils.getCaseOrder(sort);
  }

  /**
   * Returns the data-type value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the data-type value for the specified MSLSort object.
   * @deprecated use ModelUtils.getDataType(SortDesignator sort).
   */
  public String getDataType(SortDesignator sort)
  {
    return ModelUtils.getDataType(sort);
  }

  /**
   * Retrieves the custom function refinement from the given mapping 
   * if the mapping has one.
   * @param mapping a Mapping object.
   * @return a CustomFunctionRefinement if the mapping has one or null otherwise.
   * @deprecated use ModelUtils.getCustomRefinement(Mapping mapping).
   */
  public CustomFunctionRefinement getCustomRefinement(Mapping mapping)
  {
    return ModelUtils.getCustomRefinement(mapping);
  }

  /**
   * Determines if a custom refinement's code is external or not.
   * @param custom the custom refinement to test.
   * @return true if a custom refinement's code is external, false otherwise.
   * @deprecated use ModelUtils.isExternal(CustomFunctionRefinement custom).
   */
  public boolean isExternal(CustomFunctionRefinement custom)
  {
    return ModelUtils.isExternal(custom);
  }
  
  /**
   * Returns the child mappings of the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of child mappings for the specified mapping.
   * @deprecated . 
   */
  protected List getChildMappings(Mapping mapping)
  {
	List list = new ArrayList();
    List childMappings = mapping.getNested();
  
	for (Iterator i = childMappings.iterator(); i.hasNext(); )
	{
	  Object o = i.next();
	  if (o instanceof Mapping)
	  {  
	    list.add(o);
	  }  
	}       
	return list;
  }
  
  /**
   * @param currentDesignator
   * @param contextDesignator
   * @return
   * @deprecated use ModelUtils.isAncestor(MappingDesignator, MappingDesignator).
   */
  public static boolean isAncestor(MappingDesignator currentDesignator, MappingDesignator contextDesignator)
  {
    return ModelUtils.isAncestor(currentDesignator, contextDesignator);
  }
}
