/*******************************************************************************
 *  Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp.  2001, 2006.  All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/package com.ibm.ccl.mapping.codegen.util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EObject;

import com.ibm.ccl.mapping.Code;
import com.ibm.ccl.mapping.ConditionRefinement;
import com.ibm.ccl.mapping.CustomFunctionRefinement;
import com.ibm.ccl.mapping.FunctionRefinement;
import com.ibm.ccl.mapping.GroupRefinement;
import com.ibm.ccl.mapping.InlineRefinement;
import com.ibm.ccl.mapping.Mapping;
import com.ibm.ccl.mapping.MappingDesignator;
import com.ibm.ccl.mapping.MoveRefinement;
import com.ibm.ccl.mapping.RefinableComponent;
import com.ibm.ccl.mapping.SemanticRefinement;
import com.ibm.ccl.mapping.SortDesignator;
import com.ibm.ccl.mapping.SortRefinement;
import com.ibm.ccl.mapping.SubmapRefinement;
import com.ibm.ccl.mapping.codegen.AnnotationConstants;
import com.ibm.ccl.mapping.codegen.XSLTConstants;

public class ModelUtils 
{
  /**
   * Return a list of nested mappings for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of nested mappings.
   */
  public static List getNestedMappings(Mapping mapping)
  {
	List result = new ArrayList();
	if (mapping != null)
	{
	  List childMappings = mapping.getNested();
	  for (Iterator i = childMappings.iterator(); i.hasNext();)
	  {
	    RefinableComponent refinableComponent = (RefinableComponent)i.next();
	    if (refinableComponent instanceof Mapping)
	    {
	  	  Mapping childMapping = (Mapping)refinableComponent;
		  result.add(childMapping);
	  	  List nestedMappings = getNestedMappings(childMapping);
		  if ((nestedMappings != null) && (!nestedMappings.isEmpty()))
		  {
		    result.addAll(nestedMappings);
		  }
		}
	  }
	}
	return result;
  }

  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param context the current mapping context
   * @param eobject the target object.
   * @return the mapping associated with the target object.
   */
  public static Mapping getMapping(Mapping context, EObject eobject)
  {
	Mapping result = null;
	List mappings = getInputMappings(context, eobject);
	if ((mappings != null) && (!mappings.isEmpty()))
	{
	  result = (Mapping)mappings.get(0);
	}
	return result;
  }
  
  /**
   * Return the mapping associated with the target object.
   * 
   * Note if multiple mappings, the first mapping is returned.
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mapping associated with the target object.
   */
  public static Mapping getMapping(List nestedMappings, EObject eobject)
  {
	Mapping result = null;
	if (nestedMappings != null)
	{
	  int size = nestedMappings.size()-1;
	  for (int i = size; i>=0; i--)
	  {
	    Mapping contextMapping = (Mapping)nestedMappings.get(i);
	    List mappings = getInputMappings(contextMapping, eobject);
	    if ((mappings != null) && (!mappings.isEmpty()))
	    {
	      result = (Mapping)mappings.get(0);
	      break;
	    }
	  }
	}
	return result;
  }

  /**
   * Return the mappings associated with the target object.
   * 
   * @param nestedMappings context mappings
   * @param ebbject the target object.
   * @return the mappings associated with the target object.
   */
  public static List getMappings(List nestedMappings, EObject eobject, Stack mappingDeclarationContext)
  {
	List result = new ArrayList();
	if (nestedMappings != null)
	{
	  int size = nestedMappings.size()-1;
	  for (int i = size; i>=0; i--)
	  {
	  	Mapping contextMapping = (Mapping)nestedMappings.get(i);
	  	List targetContext = getTargetContext(mappingDeclarationContext);
	   	List mappings = getInputMappings(contextMapping, eobject, targetContext);
	    if ((mappings != null) && (!mappings.isEmpty()))
	    {
	      result = mappings;
	      break;
	    }
	  }
	}
	return result;
  }

  public static List getTargetContext(Stack mappingDeclarationContext) 
  {
	List result = new ArrayList();
	if (mappingDeclarationContext != null)
	{
	  for (Iterator i = mappingDeclarationContext.iterator(); i.hasNext();)
	  {
	    result.add(i.next());
	  }
	}
	return result;
  }

  public static List getInputMappings(Mapping baseMapping, EObject target, List targetContext)
  {
    List list = new ArrayList();
	List mappings = getInputMappings(baseMapping, target);
    if ((mappings != null) && (!mappings.isEmpty()))
    {
  	  for (Iterator i = mappings.iterator(); i.hasNext(); )
	  {
  		Mapping mapping = (Mapping)i.next();
	    List mappingContext = getMappingContextList(getPrimaryTargetDesignator(baseMapping), getPrimaryTargetDesignator(mapping));
	    int index = targetContext.size() - mappingContext.size();
	    if (((index == 0) && targetContext.equals(mappingContext)) ||
	    	((index > 0) && targetContext.subList(index,targetContext.size()).equals(mappingContext)))  		
	    {
          list.add(mapping);
	    }
	  }
    }
    return list;
  }

  public static List getInputMappings(Mapping baseMapping, EObject target)
  {
    List list = new ArrayList();
	if (baseMapping != null)
	{
	  List childMappings = baseMapping.getNested();
	  for (Iterator i = childMappings.iterator(); i.hasNext(); )
	  {
	    Object o = i.next();
	    if (o instanceof Mapping)
	    {
	   	  Mapping mapping = (Mapping)o;
	      List designators = mapping.getOutputs();
	      for (Iterator j = designators.iterator(); j.hasNext();)
	      {
	        MappingDesignator designator = (MappingDesignator)j.next();
            if (designator.getObject() == target)
	        {
	          list.add(mapping);
	        }
	      }      
	    }  
	  }       
	}
	return list;
  }

  public static List getInputMappings(Mapping baseMapping, EObject ancestor, EObject target)
  {
    List list = new ArrayList();
	if ((baseMapping != null) && (target != null))
	{
	  List childMappings = baseMapping.getNested();
	  for (Iterator i = childMappings.iterator(); i.hasNext(); )
	  {
	    Object o = i.next();
	    if (o instanceof Mapping)
	    {
	      Mapping mapping = (Mapping)o;
	      MappingDesignator designator = getPrimaryTargetDesignator(mapping);
	      EObject candidate = getPrimaryTarget(mapping);
          if ((candidate == target) && ((ancestor == null) || isCommonAncestor(designator, ancestor)))
	      {
            list.add(mapping);
          }
	    }  
	  }       
    }
	return list;
  }

  public static List getMappingContextList(MappingDesignator base, MappingDesignator target)
  {
    List result = new ArrayList();
    if ((base != null) && (target != null)) 
    {
	  result.add(target.getObject());	
	  MappingDesignator parent = target.getParent();
	  boolean baseFound = false;
	  while ((parent != null) && !baseFound)
	  {
	    result.add(0, parent.getObject());
   		if(base == parent) 
  		  baseFound = true;
   		else
   		  parent = parent.getParent();
      }
	  if (!baseFound)
		result = Collections.EMPTY_LIST;
    }
    return result;
  }

  public static boolean isAncestor(MappingDesignator currentDesignator, MappingDesignator contextDesignator)
  {
    boolean isAncestor = false;
    if ((currentDesignator != null) && (contextDesignator != null))
    {
      MappingDesignator parent = currentDesignator.getParent();
      while (parent != null)
      {
  	    isAncestor = (parent == contextDesignator);
  	    if (isAncestor)
  	  	  break;
  	    else
          parent = parent.getParent();
      }
    }
    return isAncestor;
  }

  public static boolean isCommonAncestor(MappingDesignator designator, EObject ancestor) 
  {
	if (ancestor == null)
	  return false;

    while (designator != null) 
	{
	  if (ancestor == designator.getObject())
        return true;

      designator = designator.getParent();
    }
	return false;
  }

  /**
   * Return the submap refinement for the specified mapping.
   * @param mapping a Mapping object.
   * @return the submap refinement for the specified mapping.
   */
  public static SubmapRefinement getSubmapRefinement(Mapping mapping)
  {
    SubmapRefinement result = null;
	if (mapping != null)
	{
	  List refinements = mapping.getRefinements();
	  if (refinements != null)
	  {
	  	for (Iterator i = refinements.iterator(); i.hasNext();)
	  	{
	  	  SemanticRefinement refinement = (SemanticRefinement)i.next();
	  	  if (refinement instanceof SubmapRefinement)
	  	  {
	  	  	result =(SubmapRefinement)refinement;
	  	    break;
	  	  }
	  	}
	  }
	}
	return result;
  }

  /**
   * Returns true if the specified mapping is an inline refinement.
   * @param mapping a Mapping object.
   * @return true if the specified mapping is an inline refinement.
   */
  public static boolean hasInlineRefinement(Mapping mapping) 
  {
    boolean result = false;
	if (mapping != null)
	{
	  List refinements = mapping.getRefinements();
	  if (refinements != null)
	  {
	  	for (Iterator i = refinements.iterator(); i.hasNext();)
	  	{
	  	  SemanticRefinement refinement = (SemanticRefinement)i.next();
	  	  if (refinement instanceof InlineRefinement)
	  	  {
	  	  	result = true;
	  	    break;
	  	  }
	  	}
	  }
	}
	return result;
  }
  
  
  /**
   * Return the ref name of the submap refinement 
   * @param submap a submap refinement.
   * @return the ref name of the submap refinement.
   */
  public static String getRefName(SubmapRefinement submap)
  {
   	String result = null;
   	if (submap != null)
   	{
   	  result = submap.getRefName();
   	}
   	return result;
  }
	
  /**
   * Return the sources for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of sources for the specified mapping.
   */
  public static List getSources(Mapping mapping)
  {
	List result = new ArrayList();
	if (mapping != null)
	{
	  List designators = mapping.getInputs();
	  if (designators != null)
	  {
	    for (Iterator i = designators.iterator(); i.hasNext();)
	    {
	      MappingDesignator designator = (MappingDesignator)i.next();
	      EObject eobject = designator.getObject();
          if (eobject != null)
          {
          	result.add(eobject);
          }
		}
	  }
	}
    return result;
  }
	
  /**
   * Return the target for the specified mapping.
   * Note if multiple target objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   */
  public static EObject getPrimaryTarget(Mapping mapping)
  {
	EObject result = null;
	if ((mapping != null) && (mapping.getOutputs().size() > 0))
	{
	  MappingDesignator designator = (MappingDesignator) mapping.getOutputs().get(0);
	  result = designator.getObject();
	}
	return result;
  }

  /**
   * Return the source for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source for the specified mapping.
   */
  public static EObject getPrimarySource(Mapping mapping)
  {
	EObject result = null;
	if ((mapping != null) && (mapping.getInputs().size() > 0))
	{
	  MappingDesignator designator = (MappingDesignator) mapping.getInputs().get(0);
	  result = designator.getObject();
	}
	return result;
  }

  /**
   * Return the target for the specified mapping.
   * @param mapping a Mapping object.
   * @return the target for the specified mapping.
   */
  public static EObject getTarget(Mapping mapping)
  {
	EObject result = null;
	if (mapping != null)
	{
	  List designators = mapping.getOutputs();
	  if ((designators != null) && (!designators.isEmpty()))
	  {
	    result = ((MappingDesignator)designators.get(0)).getObject();
	  }
	}
    return result;
  }
  
  /**
   * Return the source designators for the specified mapping.
   * @param mapping a Mapping object.
   * @return the list of source designators for the specified mapping.
   */
  public static List getSourceDesignators(Mapping mapping)
  {
    List result = new ArrayList();
	if (mapping != null)
	{
	  List designators = mapping.getInputs();
	  if (designators != null)
	  {
	    for (Iterator i = designators.iterator(); i.hasNext();)
	    {
	      result.add((MappingDesignator)i.next());
		}
	  }
	}
	return result;
  }
  
  /**
   * Return the target designator for the specified mapping.
   * Note if multiple target objects, the first target object is returned.
   *    
   * @param mapping a Mapping object.
   * @return the target designator for the specified mapping.
   */
  public static MappingDesignator getPrimaryTargetDesignator(Mapping mapping)
  {
	MappingDesignator result = null;
	if (mapping != null)
	{
	  List designators = mapping.getOutputs();
	  if ((designators != null) && (!designators.isEmpty()))
	  {
	    result = (MappingDesignator)designators.get(0);
	  }
    }
	return result;
  }
  
  /**
   * Return the source designator for the specified mapping.
   * Note if multiple source objects, the first source object is returned.
   * 
   * @param mapping a Mapping object.
   * @return the source designator for the specified mapping.
   */
  public static MappingDesignator getPrimarySourceDesignator(Mapping mapping)
  {
	MappingDesignator designator = null;
	if ((mapping != null) && (mapping.getInputs().size() > 0))
	{
	  designator = (MappingDesignator) mapping.getInputs().get(0);
	}
	return designator;
  }

  /**
   * Return the base designator for the specified mapping that is the 
   * ancestor of the specified designator.
   * 
   * @param mapping a Mapping object.
   * @param designator a descendant of the base designator.
   * @return the source designator for the specified mapping.
   */
  public static MappingDesignator getBaseSourceDesignator(Mapping mapping, MappingDesignator designator)
  {
	MappingDesignator result = null;
	if (mapping != null)
	{
	  if (mapping.getInputs().size() == 1)
	  {
	    result = (MappingDesignator) mapping.getInputs().get(0);
	  }
	  else
	  {
	  	for (Iterator i = mapping.getInputs().iterator(); i.hasNext();)
	  	{
	  	  MappingDesignator candidate = (MappingDesignator) i.next();
	   	  if ((candidate == designator) ||isAncestor(designator, candidate))
	   	  {
	   	    result = candidate;
	   	    break;
	   	  }
	   	}
	  }
	}
	return result;
  }
  
  /**
   * Returns true if the specified mapping contains a condition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a condition.
   */
  public static boolean hasConditionRefinement(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
 	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	        
	      if (condition.getCode() != null)
	      {  
	        result =true;
	        break;
	      }
	    } 
	  }
	}
	return result;
  }

  /**
   * Returns a  condition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a condition for the specified mapping.
   */
  public static ConditionRefinement getConditionRefinement(Mapping mapping)
  {
	ConditionRefinement result = null;
	if (mapping != null)
	{
      for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	        
	      if (condition.getCode() != null)
	      {  
	        result = condition;
	        break;
	      }
	    } 
	  }
	}
	return result;
  }

  /**
   * Returns the test value of the specified condition.
   * @param condition a ConditionRefinement object.
   * @return the test value of the specified condition.
   */
  public static String getTestValue(ConditionRefinement condition)
  {
	String result = null;
	if (condition != null)
	{
	  Code code = condition.getCode();
	  if (code != null)
	  {
	  	if (code.isInline().booleanValue())
	  	{
	      result = code.getInternalCode();
	  	}
	  	else
	  	{
	  	  result = code.getExternalCode();
	  	}
	  }
	}
	return result;
  }
	
  /**
   * Returns true if the specified mapping contains inner scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains inner scope conditions.
   */
  public static boolean hasInnerScopeCondition(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	      EMap annotations = condition.getAnnotations();
	      if ((annotations != null) && (!annotations.isEmpty()))
	      {  
       	    String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	        if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_INNER)))
	        {
	          result = true;
	          break;
	        }
	      }
	      else
	      {
	        // default is to treat condition as an inner condition 
	        result = true;
	        break;
	      }
	    }
	  }
	}
	return result;
  }

  /**
   * Returns a list of inner scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope conditions for the specified mapping.
   */
  public static ConditionRefinement getInnerScopeCondition(Mapping mapping)
  {
	ConditionRefinement result = null;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	      EMap annotations = condition.getAnnotations();
	      if ((annotations != null) && (!annotations.isEmpty()))
	      {  
	   	    String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	        if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_INNER)))
	        {
	          result = condition;
	          break;
	        }
	      }
	      else
	      {
	        // default is to treat condition as an inner condition 
	        result = condition;
	        break;
	      }
	    }
	  }
	}
	return result;
  }
  
  /**
   * Returns true if the specified mapping contains outer scope conditions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains outer scope conditions.
   */
  public static boolean hasOuterScopeCondition(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
 	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	      EMap annotations = condition.getAnnotations();
	      if ((annotations != null) && (!annotations.isEmpty()))
	      {  
	   	    String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	        if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_OUTER)))
	        {
	          result = true;
	          break;
	        }
	      }
	    } 
	  }
	}
	return result;
  }

  /**
   * Returns a list of outer scope conditions for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope conditions for the specified mapping.
   */
  public static ConditionRefinement getOuterScopeCondition(Mapping mapping)
  {
	ConditionRefinement result = null;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof ConditionRefinement)
	    {
	      ConditionRefinement condition = (ConditionRefinement)refinement;
	      EMap annotations = condition.getAnnotations();
	      if ((annotations != null) && (!annotations.isEmpty()))
	      {  
	   	    String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	        if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_OUTER)))
	        {
	          result = condition;
	          break;
	        }
	      }
	    } 
	  }
	}
	return result;
  }

  /**
   * Returns true if the specified mapping contains grouping definitions.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains grouping definitions.
   */
  public static boolean hasGroupRefinement(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof GroupRefinement)
	    {
	      GroupRefinement grouping = (GroupRefinement)refinement;
	      List fields = grouping.getFields();
	      if ((fields != null) && (!fields.isEmpty()))
	      {
	        result = true;
	        break;
	      }
	    }
	  }
	}
	return result;
  }

  /**
   * Returns a Grouping definition for the specified mapping.
   * @param mapping a Mapping object.
   * @return a Grouping definition for the specified mapping.
   */
  public static GroupRefinement getGroupRefinement(Mapping mapping)
  {
	GroupRefinement result = null;
	if (mapping != null)
	{  
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof GroupRefinement)
	    {
	      GroupRefinement grouping = (GroupRefinement)refinement;
	      List fields = grouping.getFields();
	      if ((fields != null) && (!fields.isEmpty()))
	   	  {
	        result = grouping;
	        break;
	      }
	    }
	  }
	}
	return result;
  }

  /**
   * Returns the name for the specified MSLGrouping object.
   * @param grouping a MSLGrouping object.
   * @return the name for the specified MSLGrouping object.
   */
  public static String getGroupingName(GroupRefinement grouping)
  {
	return "group1"; //$NON-NLS-1$ 
  }

  /**
   * Returns true if the specified mapping contains a function.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains  function.
   */
  public static boolean hasFunctionRefinement(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof FunctionRefinement)
	    {
	  	  result = true;
	   	  break;
	    }
	  }
	}
	return result;
  }

  /**
   * Returns the function for the specified mapping.
   * @param mapping  a Mapping object.
   * @return the function for the specified mapping.
   */
  public static FunctionRefinement getFunctionRefinement(Mapping mapping)
  {
	FunctionRefinement result = null;
    if (mapping != null)
    {  
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof FunctionRefinement)
	    {
	      result = (FunctionRefinement)refinement;
		  break;
	    }
	  }
	}
	return result;
  }

  /**
   * Returns true if the specified mapping contains a Sort definition.
   * @param mapping a Mapping object.
   * @return true if the specified mapping contains a Sort definition.
   */
  public static boolean hasSortRefinement(Mapping mapping)
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof SortRefinement)
	    {
	      SortRefinement sort = (SortRefinement)refinement;
	        
	      List fields = sort.getFields();
	      if ((fields != null) && (!fields.isEmpty()))
	      {  
	        result =true;
	        break;
	      } 
	    }
	  }
	}
	return result;
  }

  /**
   * Returns a list of sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of sorts for the specified mapping.
   */
  public static List getSorts(Mapping mapping)
  {
	List result = new ArrayList();
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof SortRefinement)
	    {
	      SortRefinement sort = (SortRefinement)refinement;
	      List fields = sort.getFields();
	      for (Iterator j = fields.iterator(); j.hasNext();)
	      {
	        result.add(j.next());
	      } 
	      break;
	    }
	  }
	}
	return result;
  }

  /**
   * Returns a list of inner scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of inner scope sorts for the specified mapping.
   */
  public static List getInnerScopeSorts(Mapping mapping)
  {
	List result = new ArrayList();
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof SortRefinement)
	    {
	      SortRefinement sortRefinement = (SortRefinement)refinement;
	      List fields = sortRefinement.getFields();
	      for (Iterator j = fields.iterator(); j.hasNext();)
	      {
	        SortDesignator sort = (SortDesignator)j.next();
	        EMap annotations = sort.getAnnotations();
	        if (annotations != null)
	        {  
	      	  String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	          if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_INNER)))
	          {
	            result.add(sort);
	          }
	        }
	      }
	    } 
	  }
	}
	return result;
  }
	
  /**
   * Returns a list of outer scope sorts for the specified mapping.
   * @param mapping a Mapping object.
   * @return a list of outer scope sorts for the specified mapping.
   */
  public static List getOuterScopeSorts(Mapping mapping)
  {
	List result = new ArrayList();
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof SortRefinement)
	    {
	      SortRefinement sortRefinement = (SortRefinement)refinement;
	      List fields = sortRefinement.getFields();
	      for (Iterator j = fields.iterator(); j.hasNext();)
	      {
	        SortDesignator sort = (SortDesignator)j.next();
	        EMap annotations = sort.getAnnotations();
	        if (annotations != null)
	        {  
	      	  String scope = (String)annotations.get(AnnotationConstants.SCOPE_KEY);
	          if ((scope != null) && (scope.equalsIgnoreCase(AnnotationConstants.SCOPE_VALUE_OUTER)))
	          {
	            result.add(sort);
	          }
	        }
	      }
	    } 
	  }
	}
	return result;
  }
	
  /**
   * Returns the order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the order value for the specified MSLSort object.
   */
  public static String getOrder(SortDesignator sort)
  {
	String result = null;
	if (sort != null)
	{
	  String order = sort.getModifier();
	  if ((order != null) && (order.equalsIgnoreCase(AnnotationConstants.ORDER_VALUE_DESCENDING)))
	  {  
	    result = XSLTConstants.ORDER_VALUE_DESCENDING;
	  }
	} 
	return result;
  }

  /**
   * Returns the case-order value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the case-order value for the specified MSLSort object.
   */
  public static String getCaseOrder(SortDesignator sort)
  {
	String result = null;
	if (sort != null)
	{
	  EMap annotations = sort.getAnnotations();
	  if (annotations != null)
	  {  
	    String caseOrder = (String)annotations.get(AnnotationConstants.CASE_ORDER_KEY);
	    if (caseOrder != null)
	    {
	      if (caseOrder.equalsIgnoreCase(AnnotationConstants.CASE_ORDER_VALUE_UPPER_FIRST))
	          result = XSLTConstants.CASE_ORDER_VALUE_UPPER_FIRST;
	      else if (caseOrder.equalsIgnoreCase(AnnotationConstants.CASE_ORDER_VALUE_LOWER_FIRST))
	          result = XSLTConstants.CASE_ORDER_VALUE_LOWER_FIRST;
	    }
	  }
	} 
	return result;
  }

  /**
   * Returns the data-type value for the specified MSLSort object.
   * @param sort a MSLSort object.
   * @return the data-type value for the specified MSLSort object.
   */
  public static String getDataType(SortDesignator sort)
  {
	String result = null;
	if (sort != null)
	{
	  EMap annotations = sort.getAnnotations();
	  if (annotations != null)
	  {  
	    String dataType = (String)annotations.get(AnnotationConstants.DATA_TYPE_KEY);
	    if ((dataType != null) && (dataType.equalsIgnoreCase(AnnotationConstants.DATA_TYPE_VALUE_NUMBER)))
	    {
	      result = AnnotationConstants.DATA_TYPE_VALUE_NUMBER;
	    }
	  }
	} 
	return result;
  }

  /**
   * Retrieves the custom function refinement from the given mapping 
   * if the mapping has one.
   * @param mapping a Mapping object.
   * @return a CustomFunctionRefinement if the mapping has one or null otherwise.
   */
  public static CustomFunctionRefinement getCustomRefinement(Mapping mapping)
  {
	if (mapping == null)
	{
	  return null;
	}
	    
	CustomFunctionRefinement customRefinement = null;

	List refinements = mapping.getRefinements();
	Iterator iterator = refinements.iterator();
	while (iterator.hasNext())
	{
	  SemanticRefinement refinement = (SemanticRefinement)iterator.next();
	  if (refinement instanceof CustomFunctionRefinement)
	  {
	    customRefinement = (CustomFunctionRefinement)refinement;
	    break;
	  }
	}
    return customRefinement;
  }
  
  public static boolean hasMoveRefinement(Mapping mapping) 
  {
	boolean result = false;
	if (mapping != null)
	{
	  for (Iterator i = mapping.getRefinements().iterator(); i.hasNext(); )
	  {
	    Object refinement = i.next();
	    if (refinement instanceof MoveRefinement)
	    {
	      result = true;
	      break;
	    }
	  }
	}
	return result;
  }

  
  /**
   * Determines if a custom refinement's code is external or not.
   * @param custom the custom refinement to test.
   * @return true if a custom refinement's code is external, false otherwise.
   */
  public static boolean isExternal(CustomFunctionRefinement custom)
  {
	boolean result = false;
	if (custom != null)
	{
	  Code code = custom.getCode();
	    
	  if (code != null && !code.isInline().booleanValue())
	  {
	    result = true;
	  }
	}
	return result;
  }
  
  public static MappingDesignator getRootDesignator(MappingDesignator designator) 
  {
	if (designator == null)
		return null;
	if (designator.getParent() == null)
		return designator;

	return getRootDesignator(designator.getParent());
  }

  public static boolean isContentMapped(Mapping contextMapping, List targetContext)
  {
	boolean result = false;
	List nestedMappings = getNestedMappings(contextMapping);
	if ((nestedMappings != null) && (targetContext != null))
	{
	  for (Iterator i = nestedMappings.iterator(); i.hasNext();)
	  {
	    Mapping mapping = (Mapping)i.next();
	    List mappingContext = getMappingContextList(getPrimaryTargetDesignator(contextMapping), getPrimaryTargetDesignator(mapping));
        if (!mappingContext.isEmpty())
        {
	      int targetIndex = targetContext.indexOf(mappingContext.get(0));
	      int mappingIndex = mappingContext.indexOf(targetContext.get(targetContext.size()-1));
	      if ((targetIndex != -1) && (mappingIndex != -1))
	      {
	        List subTargetContextList = targetContext.subList(targetIndex, targetContext.size());
	        List subMappingContextList = mappingContext.subList(0, mappingIndex + 1);
	        if (!subTargetContextList.isEmpty() && subTargetContextList.equals(subMappingContextList))
	        {
	       	  result = true;
	       	  break;
	        }
	      }
       	}
	  }
	}
	return result;
  }

  public static boolean isExternalCustomDefinition(CustomFunctionRefinement customFunctionRefinement) 
  {
	return false;
  }
}
